// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package xflags

import (
	"fmt"
	"github.com/stretchr/testify/assert"
	"strconv"
	"testing"
)

func TestWrongDefinitionType(t *testing.T) {
	c := New("root", 2)
	c.Parse([]string{"test"})
	c.Execute()
	assert.True(t, c.HasErrors())
}

type testExecuteCommandStruct struct {
	Command1 struct {
	} `command:"command1" description:"Command 1" callback:"command1Callback" `
	Command2 struct {
		Command3 struct {
		} `command:"command3" description:"Command 3" callback:"command3Callback" call:"DoCmd3"`
	} `command:"command2" description:"Command 2" callback:"command2Callback" `
}

func (c *testExecuteCommandStruct) DoCmd3(s *Settings[testExecuteCommandStruct]) {
}

func (c *testExecuteCommandStruct) command1Callback(args []string) {
	fmt.Println("command1Callback", args)
}

func TestExecute1(t *testing.T) {
	c := New("root", testExecuteCommandStruct{})
	c.Parse([]string{"command2", "command3", "commandX"})
	c.Execute()
	assert.False(t, c.HasErrors())
}

type testExecutionStruct struct {
	callbackCounter int `ignore:"true"`

	// for tag proxy see TestFlagCopyToProxy
	Global1  bool `short:"a" long:"global1" description:"Global 1" map:"ValGlobal1"`
	Global2  bool `short:"b" long:"global2" description:"Global 2"`
	Command1 struct {
		Command1Flag1 bool `short:"c" long:"command1flag1" description:"Command 1 Flag 1"`
		Command1Flag2 bool `short:"d" long:"command1flag2" description:"Command 1 Flag 2" map:"ValCommand1Flag2"`
		Command2      struct {
			Command2Flag1 bool `short:"e" long:"command2flag1" description:"Command 2 Flag 1"`
			Command2Flag2 bool `short:"f" long:"command2flag2" description:"Command 2 Flag 2"`
		} `command:"command2" description:"Command 2" call:"Command2Callback" `
	} `command:"command1" description:"Command 1" call:"Command1Callback" `
	Command3 struct {
		Command3Flag1 bool `short:"g" long:"command3flag1" description:"Command 3 Flag 1"`
	} `command:"command3" description:"Command 3" call:"Command3Callback" `
}

func (c *testExecutionStruct) Command1Callback(s *Settings[testExecutionStruct]) {
	s.definitions.callbackCounter++
}

func (c *testExecutionStruct) Command2Callback(s *Settings[testExecutionStruct]) {
	s.definitions.callbackCounter++
}

func (c *testExecutionStruct) Command3Callback(s *Settings[testExecutionStruct]) {
	s.definitions.callbackCounter++
}

type testExecutionTestCases[C any] struct {
	name        string
	args        []string
	targetValue int
	hasErrors   bool
}

func TestExec(t *testing.T) {

	tests := []testExecutionTestCases[testExecutionStruct]{
		{
			name:        "test",
			args:        []string{"command1", "-c"},
			targetValue: 1,
			hasErrors:   true,
		},
		{
			name:        "test",
			args:        []string{"command1", "command2", "-e"},
			targetValue: 1,
			hasErrors:   true,
		}, {
			name:        "test",
			args:        []string{"-a", "command3"},
			targetValue: 1,
			hasErrors:   true,
		},
	}

	for i, tt := range tests {
		t.Run(tt.args[0], func(t *testing.T) {
			settings := New(tt.name, testExecutionStruct{})
			assert.NotNil(t, settings)

			if settings.definitions.callbackCounter != 0 {
				t.Error("Callback counter should be 0")
			}

			settings.Parse(tt.args)
			settings.Execute()

			if !tt.hasErrors && settings.HasErrors() {
				t.Error("run " + strconv.Itoa(i) + ": should not have errors but has: " + strconv.Itoa(len(settings.Errors())))
				t.Log(settings.Errors())
				return
			}

			if settings.definitions.callbackCounter != tt.targetValue {
				t.Error("Callback counter should be " + strconv.Itoa(tt.targetValue) + " but is " + strconv.Itoa(settings.definitions.callbackCounter))
			}

			//tt.test(s)
		})
	}
}
