// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package xflags

import (
	"flag"
	"gitlab.schukai.com/oss/libraries/go/utilities/pathfinder.git"
	"reflect"
	"strconv"
	"strings"
)

// SetMappedObject sets the shadow struct for the flag configuration.
func (s *Settings[C]) SetMappedObject(proxy Copyable) *Settings[C] {

	if reflect.TypeOf(proxy).Kind() != reflect.Ptr {
		s.errors = append(s.errors, ShadowMustBePointerError)
		return s
	}

	if reflect.TypeOf(proxy).Elem().Kind() != reflect.Struct {
		s.errors = append(s.errors, ShadowMustBePointerError)
		return s
	}

	s.proxy = proxy
	return s
}

// Copyable is the interface for the proxy struct.
type Copyable interface {
	Copy(map[string]any)
}

func (s *Settings[C]) assignValues(c cmd[C]) {
	flgs := c.flagSet
	flgs.Visit(func(f *flag.Flag) {

		name := f.Name
		stringValue := f.Value.String()

		k, ok := c.tagMapping[name]
		if !ok {
			s.errors = append(s.errors, newUnknownFlagError(name))
			return
		}

		pa := append(c.valuePath, k)
		p := strings.Join(pa, ".")

		q, err := pathfinder.GetValue(&s.definitions, p)
		if err != nil {
			s.errors = append(s.errors, err)
			return
		}

		if q == nil {
			s.errors = append(s.errors, newUnknownFlagError(name))
			return
		}

		typeOf := reflect.TypeOf(q)

		switch typeOf.Kind() {
		case reflect.String:
			err = pathfinder.SetValue(&s.definitions, p, stringValue)
		case reflect.Int:
			intVar, err := strconv.Atoi(stringValue)
			if err != nil {
				s.errors = append(s.errors, err)
				return
			}
			err = pathfinder.SetValue(&s.definitions, p, intVar)
		case reflect.Bool:
			boolVar, err := strconv.ParseBool(stringValue)
			if err != nil {
				s.errors = append(s.errors, err)
				return
			}
			err = pathfinder.SetValue(&s.definitions, p, boolVar)
		case reflect.Slice:

			switch typeOf.Elem().Kind() {
			case reflect.String:
				qs := q.(StringFlags)
				stringSliceValue := f.Value.(*StringFlags)
				for _, v := range *stringSliceValue {
					err = qs.Set(v)
					if err != nil {
						s.errors = append(s.errors, err)
						return
					}
				}
				err = pathfinder.SetValue(&s.definitions, p, qs)
			default:
				err = pathfinder.SetValue(&s.definitions, p, stringValue)
			}

		}

		if err != nil {
			s.errors = append(s.errors, err)
		}

		if c.proxyMapping[k] != "" {
			p = c.proxyMapping[k]
			s.mapping[p] = stringValue
		}

	})

}
